<?php
/* Heizungssteuerung - VL Temperatur Berechnung */

include "eh_global.ips.php";
$ds->setname("heizung_wp");
$ds->Log("?  ".str_pad(IPS_GetName($_IPS['SELF']), 80, "=", STR_PAD_RIGHT));

// Einstellungen
$HYSTERESE      = 0.6;
$VL_OFF         = 20;  // VL fr Heizung Aus
$VL_ABSENKUNG   = 3;   // Nachtabsenkung wenn drauen sehr kalt. (s.u.)
$HEIZ_GRENZTEMP = GetValue(51201);  // Ab welcher AT wird nicht mehr geheizt

// Variablen
$idARCHIVE   = 26472;
$idTempOut   = 59388; // DEFAULT

$idVL        = 46699; // Aktueller VL der WP
$idVLSoll    = 10517; // Soll VL Temp

// Temperaturen der Rume packen mit (Soll und IST), die fr Heizung bercksichtigt werden sollen.
$temps[] = [ "sollID" =>37728, 
             "istID"   => 47087];

$temps[] = [ "sollID" =>49334, 
             "istID"  => 43090];

$on_state = getValue(19422); // Aktueller Status WP
$mode     = getValue(34876); // Aktueller Mode WP


// Vorlauf abhngig von Auentempertur festlegen.
$TVL[] = ["OUT" => 13,  "VL" => 37]; // 38 war
$TVL[] = ["OUT" => 10,  "VL" => 36]; //36 war
$TVL[] = ["OUT" => 8,  "VL"  => 36]; //36 war
$TVL[] = ["OUT" => 5,   "VL" => 35];
$TVL[] = ["OUT" => 0,   "VL" => 34];
$TVL[] = ["OUT" => -10, "VL" => 36];

$heat_old = $ds->heat;
$heat_cur = false;
if($on_state == 1  && ($mode == 0 || $mode == 2 || $mode == 4 || $mode == 6 )
   ){
    $heat_cur = true;
}

// Default - kein Heizen
$heat = false;

// Auentemperatur und Vorlauf ermitteln
$TempOut = GetValue($idTempOut);
$ds->log("VLT: Pumpe:  ".( ($heat_cur)? "AN" : "AUS"));

// prfen ob Temperaturen ok sind
$VLAdd = 0;
$heat_final = 0;

foreach($temps as $room){
    $name     =  IPS_GetName($room["istID"]);
    $ist      = getValue($room["istID"]);
    $soll     = getValue($room["sollID"]);
    $sollLow  = $soll - ($HYSTERESE / 1.5);    // nach unten etwas abgegrenzt
    $sollHigh = $soll + ($HYSTERESE / 2);  // nach oben etwas mehr luft
    $ds->log("VLT: Name:   $name");
    $ds->log("VLT:  ist:   $ist");
    $ds->log("VLT:  soll:  $soll");
    $ds->log("VLT:  frame: $sollLow  - $sollHigh");
    
    // Wenn zu kalt
    if($ist <= $sollLow )        $heat = true;

    // wenn zu warm
    if($ist >= $sollHigh)        $heat = false;

    // wenn Pumpe luft und ausgeschaltet werden soll und aktuell jedoch kurz vor unterer Schwelle bleibt sie an.
    if( ($ist - $sollLow < $HYSTERESE / 3)  &&
        $heat_final == 0 &&
        $heat_cur == true 
     ) $heat = true;

    // Wenn noch nicht drber und heizung an ist, bleibt sie an.
    if($ist > $sollLow && $ist < $sollHigh && $heat_cur)$heat = true;
    $heat_final += intval($heat);
    $ds->log("VLT:   > ".(($heat)?"AN" : "AUS"));
}
$heat_final = ($heat_final > 0 )? true : false;

$VLIst   = GetValue($idVL);

$H  = date("G");
$M  = date("i");
$HM = intval($H.$M);
$D  = date("N");

// VL Temperatur ################################################################

// Upper / Lower festlegen
$VLSoll =  $TVL[0]["VL"]; //minimaler VL festlegen
for($x=0; $x<count($TVL)-1; $x++){
    if($TempOut <= $TVL[$x]["OUT"] && $TempOut >= $TVL[$x+1]["OUT"]){
        // Mit Formel hergeleitet.
        $faktor = ($TempOut - $TVL[$x]["OUT"]) / ($TVL[$x]["OUT"] - $TVL[$x+1]["OUT"]);
        $VLSoll = ($TVL[$x]["VL"] - $TVL[$x+1]["VL"]) * $faktor + $TVL[$x]["VL"] ;
    }
}
if($TempOut > $TVL[0]["OUT"])$VLSoll =  $TVL[0]["VL"];
if($TempOut < $TVL[count($TVL)-1]["OUT"])$VLSoll =  $TVL[count($TVL)-1]["VL"];
$ds->log("VLT: TempOut:      $TempOut C");
$ds->log("VLT: VLSoll:       $VLSoll C");

// Temperaturen Wohnzimmer ########################################################
$VLChange = 0;
$VLSet    = 0;
$VLSoll   = $VLSoll + $VLAdd;
$VLSoll   = round($VLSoll);
$VLIst    = round($VLIst);

// Overheat aktiv? VL Soll anheben - guter COP Wert
if(getValue(15133)){
        $VLSoll = $VLSoll + 2;
        $VLSoll = ($VLSoll > 37)? 37 : $VLSoll;
        $ds->log("VLT: Overheat!, VLSoll $VLSoll C");
        
}


//kein Heizvorgang vor 16 Uhr, wenn Sonne angesagt ist und auetemperatur passt
$fc = json_decode(getValue(49719),true);
$dayTempMax = $fc["daily"][0]["temp_max"];
// Cloudcover von 9 - 16h prfen
$cnt = 0;
$clds = 0;
foreach($fc["hourly"] as $fh){
    if($fh["day"] == 0 && $fh["hour"]>9 && $fh["hour"]<16){
        $cnt++;
        $clds += $fh["clouds"];
    }
}
$clds = round($clds / $cnt);
if($dayTempMax > 10 && $clds < 70 && date("G") >= 8 && date("G") < 15 ) {
    $ds->log("VLT: Heute noch warm (clouds: $clds, Temp:$dayTempMax), keine Heizung zwischen 8 und 15 Uhr!");
    $heat_final = false;
    $heat = false;
}

//Heiz-Grenztemperatur

if($TempOut > $HEIZ_GRENZTEMP){
    $ds->log("VLT: Auentemperatur > Heizgrenztemperatur ($HEIZ_GRENZTEMP) --> AUS!");
    $heat = false;
    $heat_final = false;
}

$ds->log("VLT: -----------------------------------------");
$ds->log("VLT: VL-Set: $VLIst -> $VLSoll");
$ds->log("VLT: HEAT:   ".(($heat_final)?"AN": "AUS"));


// Klte MOdus - kein Abschalten runter modulieren
if(!$heat_final && $TempOut < -2 ){
    $ds->log("VLT: -----------------------------------------");
    $ds->log("VLT: Auentemperatur $TempOut, HEAT bleibt an!");
    $VLSoll = $VLSoll - $VL_ABSENKUNG;
    $ds->log("VLT:   Vorlauf wird abgesenkt auf $VLSoll");
    $heat_final = true;
}


// nur wenn 3x hintereindander
if(!$heat_final && $heat_cur){
    $ds->log("VLT: Heizung OFF Counter: $ds->heat_off_cnt ");
    $ds->heat_off_cnt ++;
    if($ds->heat_off_cnt >= 2 ){
        $ds->heat_off_cnt = 0;        
    } else{
        
        $heat_final = true;
        $ds->log("VLT: OFF-Counter noch nicht ausreichen, Pumpe bleibt erstmal an.");
    }    
}


if(time() - $ds->GetValue("USER_". $idVLSoll) > 3600*4 ){
    SetValue($idVLSoll, $VLSoll);
}else {
    $ds->log("VLT: VLSoll wird nicht gesetzt - Manuelle nderung hat Vorrang.");
}

$ds->heat = $heat_final;
$ds->log("VLT: heat_final: $heat_final");
$ds->TempOutOld2 = $ds->TempOutOld1;
$ds->TempOutOld1 = $TempOut;
$ds->VLSoll = $VLSoll;

$ds->save();